from __future__ import absolute_import, unicode_literals

REFBASE = 'http://docs.celeryproject.org/en/latest'
REFS = {
    'mailing-list':
        'http://groups.google.com/group/thorn-users',
    'irc-channel': 'getting-started/resources.html#irc',
    'events-guide':'http://thorn.readthedocs.io/en/latest/userguide/events.html',
    'reporting-bugs': 'contributing.html#reporting-bugs',
}
